#include "nrf_hd44780.h"
#include "nrf_delay.h"
#include "nrf_gpio.h"

void lcd_output_upper(uint8_t data)
{
    if(data & 0x80) nrf_gpio_pin_set(LCD_PIN_B7);
    else nrf_gpio_pin_clear(LCD_PIN_B7);
    if(data & 0x40) nrf_gpio_pin_set(LCD_PIN_B6);
    else nrf_gpio_pin_clear(LCD_PIN_B6);
    if(data & 0x20) nrf_gpio_pin_set(LCD_PIN_B5);
    else nrf_gpio_pin_clear(LCD_PIN_B5);
    if(data & 0x10) nrf_gpio_pin_set(LCD_PIN_B4);
    else nrf_gpio_pin_clear(LCD_PIN_B4);
}

void lcd_write8(uint8_t data)
{
    LCD_RW_WRITE;
    lcd_output_upper(data);
    LCD_PULSE_CLOCK;
    lcd_output_upper(data << 4);
    LCD_PULSE_CLOCK;
}

uint8_t lcd_read8()
{
    uint8_t retval = 0;
    LCD_RW_READ;
    LCD_REGISTER_SELECT_COMMAND;
    NRF_GPIO->DIRCLR = (1 << LCD_PIN_B7) | (1 << LCD_PIN_B6) | (1 << LCD_PIN_B5) | (1 << LCD_PIN_B4); 
    nrf_gpio_pin_set(LCD_PIN_CLK);
    nrf_delay_us(1);
    if(nrf_gpio_pin_read(LCD_PIN_B7)) retval |= 0x80;
    if(nrf_gpio_pin_read(LCD_PIN_B6)) retval |= 0x40;
    if(nrf_gpio_pin_read(LCD_PIN_B5)) retval |= 0x20;
    if(nrf_gpio_pin_read(LCD_PIN_B4)) retval |= 0x10;
    nrf_gpio_pin_clear(LCD_PIN_CLK);
    nrf_delay_us(1);
    nrf_gpio_pin_set(LCD_PIN_CLK);
    nrf_delay_us(1);
    if(nrf_gpio_pin_read(LCD_PIN_B7)) retval |= 0x08;
    if(nrf_gpio_pin_read(LCD_PIN_B6)) retval |= 0x04;
    if(nrf_gpio_pin_read(LCD_PIN_B5)) retval |= 0x02;
    if(nrf_gpio_pin_read(LCD_PIN_B4)) retval |= 0x01;
    nrf_gpio_pin_clear(LCD_PIN_CLK);
    NRF_GPIO->DIRSET = (1 << LCD_PIN_B7) | (1 << LCD_PIN_B6) | (1 << LCD_PIN_B5) | (1 << LCD_PIN_B4); 
    return retval;
}

bool lcd_busy()
{
    return (lcd_read8() & 0x80);
}

void lcd_enable_4bit_mode(bool two_line_display, bool big_characters)
{
    uint8_t cmd_byte = LCD_CMD_FUNCTION_SET;
    cmd_byte |= (two_line_display ? 0x08 : 0x00);
    cmd_byte |= (big_characters ? 0x04 : 0x00);
    
    //lcd_output_upper(cmd_byte);
    //LCD_PULSE_CLOCK;
    //nrf_delay_us(100);
    
    lcd_write8(cmd_byte);
  
    nrf_delay_us(100);    
}

void lcd_clear_display()
{
    LCD_REGISTER_SELECT_COMMAND;
    lcd_write8(0x01);
    while(lcd_busy());
    nrf_gpio_pin_clear(25);
}

void lcd_on_off_control(bool on, bool cursor_on, bool cursor_blink)
{
    LCD_REGISTER_SELECT_COMMAND;
    lcd_write8(0x08 | (on ? 0x04 : 0x00) | (cursor_on ? 0x02 : 0x00) | (cursor_blink ? 0x01 : 0x00));
    nrf_delay_us(50);
}

void lcd_set_cgram_address(uint8_t addr)
{
    LCD_REGISTER_SELECT_COMMAND;
    lcd_write8(0x40 | (addr & 0x3F));
    nrf_delay_us(37);
}

void lcd_set_ddram_address(uint8_t addr)
{
    LCD_REGISTER_SELECT_COMMAND;
    lcd_write8(0x80 | (addr & 0x7F));
    nrf_delay_us(37);
}

void lcd_write_string(char *string, uint32_t pos, bool lower_line)
{
    lcd_set_ddram_address(pos + (lower_line ? 0x40 : 0x00));
    while(*string)
    {
        LCD_REGISTER_SELECT_DATA;
        lcd_write8(*string++);
        nrf_delay_us(37);
    }

}

void lcd_init()
{
        nrf_gpio_range_cfg_output(LCD_PIN_BASE, LCD_PIN_BASE+7);
    NRF_GPIO->PIN_CNF[LCD_PIN_VO] = GPIO_PIN_CNF_DIR_Output << GPIO_PIN_CNF_DIR_Pos |
                            GPIO_PIN_CNF_DRIVE_H0H1 << GPIO_PIN_CNF_DRIVE_Pos;
    nrf_gpio_pin_clear(LCD_PIN_VO);
    //nrf_gpio_cfg_input(LCD_PIN_VO, NRF_GPIO_PIN_NOPULL);
    nrf_delay_us(100);
    
    nrf_gpio_pin_clear(LCD_PIN_RS); // Register select command
    nrf_gpio_pin_clear(LCD_PIN_RW); // Write mode
    nrf_gpio_pin_clear(LCD_PIN_CLK); // CLK idle
    
}
